@@ -7,7 +7,7 @@
 # Distributed under the terms of the GPL
 ###############################################################################
 ###############################################################################
-# Version 0.8.3
+# Version 0.8.5
 ###############################################################################
 ###############################################################################
 # This program recursively downloads Web sites, as it goes, it changes absolute
@@ -95,7 +95,7 @@ wm deiconify .
 #    An empty string in case of error.
 ###############################################################################
 proc InitializeDownload {url} {
-    global directories siteUrl options relocated
+    global directories siteUrl options relocated dirGetleft
     global labelTitles labelMessages
 
     if {![regexp -nocase {([^:/])(/)} $url]} {
@@ -128,12 +128,14 @@ proc InitializeDownload {url} {
         return
     }
 
-    set directories(base) [Dialogos::SelectDirectory]
+    set directories(base) [Dialogos::SelectDirectory $dirGetleft(lastDir)]
+
     if {![string compare $directories(base) ""]} {
         return
     }
-    set directories(local) $directories(base)
-    set siteUrl(base) $siteUrl(dir)
+    set dirGetleft(lastDir) [file dirname $directories(base)]
+    set directories(local)  $directories(base)
+    set siteUrl(base)       $siteUrl(dir)
 
     set result [Download $url "-" $isHtml]
     catch {wm withdraw .bajando}
@@ -207,7 +209,7 @@ proc RelativePath {path_absoluto} {
 #    url: url of the page where are going to process for links
 ###############################################################################
 proc Preprocessing {url} {
-    global directories filesDone getleftState
+    global directories filesDone getleftState charSets
 
     if [string match $url ""] return
 
@@ -233,6 +235,17 @@ proc Preprocessing {url} {
     }
 
     HtmlParser::Parsing $fileName
+    if {$getleftState(filesChosen)==0} {
+        set encoding ""
+        catch {set encoding $charSets($Ventana::Rizo::meta(charSet))}
+        if {$HtmlParser::pageEncoding!=""} {
+            set encoding $charSets($HtmlParser::pageEncoding)
+        }
+        if {$encoding!=""} {
+            HtmlParser::ChangeEncoding $encoding
+        }
+        HtmlParser::TidyDescription
+    }
     HtmlParser::FilterLinks $url
 
     if {![file exists $file.orig]} {
@@ -322,7 +335,6 @@ proc ChooseGroup {which action path} {
         set last  [expr $HtmlParser::nLinks-1]
     }
     for {set i $first} {$i<=$last} {incr i} {
-
         if {$action==0} {
             $path.enlaces$i deselect
         } else {
@@ -342,8 +354,9 @@ proc ChooseGroup {which action path} {
 #    referer: url of the page from wherer the links where taken.
 #    text: The path of the text widget
 #    map:  '1' if we are getting the site map
+#    reparse: '1' if the page ought to be reparsed, defaults to '1'.
 ###############################################################################
-proc ChangeLinksToChoose {referer text map} {
+proc ChangeLinksToChoose {referer text map {reparse 1}} {
     global filesDone
 
     $text configure -cursor watch
@@ -354,7 +367,9 @@ proc ChangeLinksToChoose {referer text map} {
     catch {unset filesDone}
     set refererFile [UrlToFile $referer]
     catch {file rename -force $refererFile.orig $refererFile}
-    Preprocessing $referer
+    if {$reparse==1} {
+        Preprocessing $referer
+    }
     LinksToChoose $text $map
 
     $text configure -state  disabled
@@ -372,39 +387,32 @@ proc ChangeLinksToChoose {referer text map} {
 #    map: '1' if we are getting the site map
 ###############################################################################
 proc LinksToChoose {text map} {
-    global siteIndex siteMap linksOk
+    global siteIndex siteMap linksOk charSets
 
     set color [$text cget -background]
 
     if {$map==1} {
         set HtmlParser::nLinks $siteIndex
     }
-puts "$HtmlParser::nLinks"
+
     for {set i 1;set pre ""} {$i<$HtmlParser::nLinks} {incr i} {
         checkbutton $text.enlaces$i -variable linksOk($i) -background $color
         set linksOk($i) 1
 
         $text window create $i.0 -window $text.enlaces$i
-        if {$map==0} {
-            set description [HtmlParser::TidyDescription \
-                    $HtmlParser::linkDescription($i)]
-        } else {
+        if {$map==1} {
+            set description $siteMap($i,2)
             for {set j 0 ; set spc "   " ; set pre ""} {$j<$siteMap($i,0)} \
                     {incr j} {
                 append pre $spc
             }
-            set description [HtmlParser::TidyDescription $siteMap($i,2)]
+        } else {
+            set description $::HtmlParser::linkDescription($i)
         }
-        if {[regexp {^[\ \t]*$} $description]} {
-            if {$map==0} {
-                set description $HtmlParser::linkFile($i)
-            } else {
-                set description $siteMap($i,1)
-            }
-        }
-        $text.enlaces$i configure -text $pre$description
 
+        $text.enlaces$i configure -text $pre$description
         $text insert insert \n
+
         bind $text.enlaces$i <Button-3> "
             regexp {(?:enlaces)(.*)} %W nada line
             $text mark set insert \$line.0
@@ -427,7 +435,7 @@ puts "$HtmlParser::nLinks"
 ###############################################################################
 proc ChooseDownloads {referer {map 0} } {
     global labelButtons labelTitles labelDialogs labelMenus
-    global dirGetleft options optionsTmp done
+    global dirGetleft options optionsTmp done charSets getleftState
 
     if {$HtmlParser::nLinks==1} "return 0"
     set elegir  [toplevel .elegir]
@@ -526,10 +534,12 @@ proc ChooseDownloads {referer {map 0} } {
             -label $labelMenus(uplinks)
     $menu.options add cascade -menu $menu.options.levels        \
             -label $labelMenus(levels)
-    $menu.options add cascade -menu $menu.options.exLevels        \
+    $menu.options add cascade -menu $menu.options.exLevels      \
             -label $labelMenus(external)
     $menu.options add cascade -menu $menu.options.filters       \
             -label $labelMenus(fileFilter)
+    $menu.options add cascade -menu $menu.options.encodings     \
+            -label $labelMenus(encodings)
     $menu.options add check   -variable options(update)         \
             -label $labelMenus(update)   -onvalue 1 -offvalue 0
     $menu.options add check   -variable options(cgi)            \
@@ -592,6 +602,21 @@ proc ChooseDownloads {referer {map 0} } {
                 ChangeLinksToChoose $referer $texto $map
             "
 
+    set menuEncodings [menu $menu.options.encodings -tearoff 0]
+    set encodings {ascii korean iso-2022-kr euc-kr iso-2022-jp iso-8859-1   \
+                   iso-8859-2 iso-8859-2 iso-8859-3 iso-8859-4 iso-8859-5   \
+                   iso-8859-6 iso-8859-7 iso-8859-8 iso-8859-9 iso-ir-159   \
+                   iso-ir-87 ibm437 ibm850 ibm852 ibm855 ibm857 ibm860     \
+                   ibm861 ibm862 ibm863 ibm864 ibm865 ibm869 koi8-r}
+    foreach code $encodings {
+        $menuEncodings add radio -variable tmp -value $code \
+                -label $code -command "
+                    Preprocessing $referer
+                    HtmlParser::ChangeEncoding \$charSets(\$tmp)
+                    ChangeLinksToChoose $referer $texto $map 0
+                "
+    }
+
     bind $elegir   <KeyPress-Prior> "$texto yview scroll -1 pages"
     bind $elegir   <KeyPress-Next>  "$texto yview scroll  1 pages"
     bind $elegir   <Key-Up>         "$texto yview scroll -1 units"
@@ -852,9 +877,12 @@ proc Download {enlace madre sufix} {
     Ventana::FileDownloading $fileName.$labelMessages(downloading) \
             $enlace $madre
     tkwait variable Ventana::Rizo::curlReport(nextFile)
+
     if {$errorCode!=""} {
-global errorInfo
-puts "Hubo un error: $errorCode: $errorInfo"
+        if {$::DEBUG==1} {
+            global errorInfo
+            puts "Hubo un error: $errorCode: $errorInfo"
+        }
         ::Herramientas::ErrorLog $fileName $enlace
         return 2
     }
@@ -1036,7 +1064,7 @@ proc ControlDownloading {urlList referer level {externalLevel 0}} {
         incr j
         if {$options(onlyHtml)==1} {
             if [regexp -nocase {(jpg)|(jpeg)|(gif)|(gz)|(tar)|(zip)|(exe)|(ps)|(pdf)|(txt)|(text)|(avi)|(ram)|(wav)} \
-                    [file extension $urlToDownload] ] {
+1                    [file extension $urlToDownload] ] {
                 set isHtml($j) 0
                 continue
             }
@@ -1427,7 +1455,7 @@ proc GetSiteMap {} {
     unset linksOk
     unset HtmlParser::linkFile
     unset HtmlParser::linkDescription
-    unset HtmlParser::linkType
+    catch {unset HtmlParser::linkType}
 
     if {$getleftState(errorFound)==1} {
         ::Herramientas::ShowErrorLog
@@ -1460,8 +1488,15 @@ proc ReadConfig { {confFile {} } } {
     } else {
         if [regexp {Windows} $tcl_platform(os)] {
             set ficherorc [file join "$dirGetleft(main)" getleft.ini]
+            set dirGetleft(lastDir) $dirGetleft(main)
         } else {
-            set ficherorc [file join $env(HOME) .Getleft]
+            set ficherorc [file join "$dirGetleft(conf)" getleft.conf]
+            set dirGetleft(lastDir) $env(HOME)
+            set temp [file join $env(HOME) .Getleft]
+            if {[file exists $temp]} {
+                set ficherorc $temp
+            }
+            set update 1
         }
     }
 
@@ -1506,11 +1541,20 @@ proc ReadConfig { {confFile {} } } {
             languages {
                 set dirGetleft(languages) $line
             }
+            lastDir {
+                set dirGetleft(lastDir) $line
+            }
         }
     }
 
     close $handle
 
+    if {[info exists update]} {
+        file delete $ficherorc
+        catch {file delete [file join $env(HOME) Geterror.log]}
+        SaveConfig
+    }
+
     if {![info exists options(lang)]} {
         set options(lang) en
     }
@@ -1533,9 +1577,9 @@ proc SaveConfig {} {
     global labelMessages labelTitles
 
     if [regexp {Windows} $tcl_platform(os)] {
-        set ficherorc [file join "$dirGetleft(main)" getleft.ini]
+        set ficherorc [file join "$dirGetleft(conf)" getleft.ini]
     } else {
-        set ficherorc [file join $env(HOME) .Getleft]
+        set ficherorc [file join "$dirGetleft(conf)" getleft.conf]
     }
 
     if [catch {open  $ficherorc w+} fichero] {
@@ -1562,6 +1606,9 @@ proc SaveConfig {} {
     puts $fichero "\n\[language\]\n"
     puts $fichero "$options(lang)"
 
+    puts $fichero "\n\[lastDir\]\n"
+    puts $fichero "$dirGetleft(lastDir)"
+
     close $fichero
 
     return
@@ -1725,6 +1772,168 @@ proc ReadLangFile {sufix} {
     return 0
 }
 
+###############################################################################
+# SetEncodings
+#    Sets the array that maps the encoding names used in http to the
+#    equivalent Tcl encodings.
+###############################################################################
+proc SetEncodings {} {
+    global charSets
+
+    set charSets(ansi_x3.4-1968)        ascii
+    set charSets(iso-ir-6)              ascii
+    set charSets(iso_646.irv:1991)      ascii
+    set charSets(iso646-us)             ascii
+    set charSets(us-ascii)              ascii
+    set charSets(us)                    ascii
+    set charSets(ibm367)                ascii
+    set charSets(cp367)                 ascii
+    set charSets(ascii)                 ascii
+
+    set charSets(ks_c_5601-1987)        ksc5601
+    set charSets(iso-ir-149)            ksc5601
+    set charSets(ks_c_5601-1989)        ksc5601
+    set charSets(ksc_5601)              ksc5601
+    set charSets(korean)                ksc5601
+
+    set charSets(iso-2022-kr)           iso2022-kr
+
+    set charSets(euc-kr)                euc-kr
+
+    set charSets(iso-2022-jp)           iso2022-jp
+
+    set charSets(iso_8859-1:1987)       iso8859-1
+    set charSets(iso-ir-100)            iso8859-1
+    set charSets(iso_8859-1)            iso8859-1
+    set charSets(iso-8859-1)            iso8859-1
+    set charSets(latin1)                iso8859-1
+    set charSets(l1)                    iso8859-1
+    set charSets(ibm819)                iso8859-1
+    set charSets(cp819)                 iso8859-1
+
+    set charSets(iso_8859-2:1987)       iso8859-2
+    set charSets(iso-ir-101)            iso8859-2
+    set charSets(iso_8859-2)            iso8859-2
+    set charSets(iso-8859-2)            iso8859-2
+    set charSets(latin2)                iso8859-2
+    set charSets(l2)                    iso8859-2
+
+    set charSets(iso_8859-3:1988)       iso8859-3
+    set charSets(iso-ir-109)            iso8859-3
+    set charSets(iso_8859-3)            iso8859-3
+    set charSets(iso-8859-3)            iso8859-3
+    set charSets(latin3)                iso8859-3
+
+    set charSets(iso_8859-4:1988)       iso8859-4
+    set charSets(iso-ir-110)            iso8859-4
+    set charSets(iso_8859-4)            iso8859-4
+    set charSets(iso-8859-4)            iso8859-4
+    set charSets(latin4)                iso8859-4
+    set charSets(l4)                    iso8859-4
+
+    set charSets(iso_8859-6:1987)       iso8859-6
+    set charSets(iso-ir-127)            iso8859-6
+    set charSets(iso_8859-6)            iso8859-6
+    set charSets(iso-8859-6)            iso8859-6
+    set charSets(ecma-114)              iso8859-6
+    set charSets(asmo-708)              iso8859-6
+    set charSets(arabic)                iso8859-6
+
+    set charSets(iso_8859-7:1987)       iso8859-7
+    set charSets(iso-ir-126)            iso8859-7
+    set charSets(iso_8859-7)            iso8859-7
+    set charSets(iso-8859-7)            iso8859-7
+    set charSets(elot_928)              iso8859-7
+    set charSets(ecma-118)              iso8859-7
+    set charSets(greek)                 iso8859-7
+    set charSets(greek8)                iso8859-7
+
+    set charSets(iso_8859-8:1988)       iso8859-8
+    set charSets(iso-ir-138)            iso8859-8
+    set charSets(iso_8859-8)            iso8859-8
+    set charSets(iso-8859-8)            iso8859-8
+    set charSets(hebrew)                iso8859-8
+
+    set charSets(iso_8859-5:1988)       iso8859-5
+    set charSets(iso-ir-144)            iso8859-5
+    set charSets(iso_8859-5)            iso8859-5
+    set charSets(iso-8859-5)            iso8859-5
+    set charSets(cyrillic)              iso8859-5
+
+    set charSets(iso_8859-9:1989)       iso8859-9
+    set charSets(iso-ir-148)            iso8859-9
+    set charSets(iso_8859-9)            iso8859-9
+    set charSets(iso-8859-9)            iso8859-9
+    set charSets(latin5)                iso8859-9
+    set charSets(l5)                    iso8859-9
+
+    set charSets(jis_x0212-1990)        jis0212
+    set charSets(x0212)                 jis0212
+    set charSets(iso-ir-159)            jis0212
+
+    set charSets(jis_x0201)             jis0201
+    set charSets(x0201)                 jis0201
+
+    set charSets(jis_c6226-1983)        jis0208
+    set charSets(iso-ir-87)             jis0208
+    set charSets(x0208)                 jis0208
+    set charSets(jis_x0208-1983)        jis0208
+
+    set charSets(ibm437)                cp437
+    set charSets(cp437)                 cp437
+    set charSets(437)                   cp437
+
+    set charSets(ibm850)                cp850
+    set charSets(cp850)                 cp850
+    set charSets(850)                   cp850
+
+    set charSets(ibm852)                cp852
+    set charSets(cp852)                 cp852
+    set charSets(852)                   cp852
+
+    set charSets(ibm855)                cp855
+    set charSets(cp855)                 cp855
+    set charSets(855)                   cp855
+
+    set charSets(ibm857)                cp857
+    set charSets(cp857)                 cp857
+    set charSets(857)                   cp857
+
+    set charSets(ibm860)                cp860
+    set charSets(cp860)                 cp860
+    set charSets(860)                   cp860
+
+    set charSets(ibm861)                cp861
+    set charSets(cp861)                 cp861
+    set charSets(861)                   cp861
+    set charSets(cp-is)                 cp861
+
+    set charSets(ibm862)                cp862
+    set charSets(cp862)                 cp862
+    set charSets(862)                   cp862
+
+    set charSets(ibm863)                cp863
+    set charSets(cp863)                 cp863
+    set charSets(863)                   cp863
+
+    set charSets(ibm864)                cp864
+    set charSets(cp864)                 cp864
+    set charSets(864)                   cp864
+
+    set charSets(ibm865)                cp865
+    set charSets(cp865)                 cp865
+    set charSets(865)                   cp865
+
+    set charSets(ibm869)                cp869
+    set charSets(cp869)                 cp869
+    set charSets(869)                   cp869
+    set charSets(cp-gr)                 cp869
+
+    set charSets(koi8-r)                koi8-r
+
+    return
+}
+
 ###############################################################################
 # MenusLabels
 #    Puts the labels into the menus.
@@ -1997,6 +2206,20 @@ if [file exists /etc/getlet.conf] {
     set dirGetleft(languages) [file join "$dirGetleft(main)" Languages]
 }
 
+if [regexp {Windows} $tcl_platform(os)] {
+    set dirGetleft(conf) $dirGetleft(main)
+} else {
+    set dirGetleft(conf) [file join $env(HOME) .getleft]
+}
+
+if {![file exists $dirGetleft(conf)]} {
+    if [catch {file mkdir $dirGetleft(conf)} error] {
+        tk_messageBox -title Error -icon error -type ok \
+                -message "Couldn't create dir:\n$error"
+        exit
+    }
+}
+
 source [file join "$dirGetleft(scripts)" Ccombinado.tcl]
 source [file join "$dirGetleft(scripts)" Ventana.tcl]
 source [file join "$dirGetleft(scripts)" Dialogos.tcl]
@@ -2007,16 +2230,17 @@ source [file join "$dirGetleft(scripts)" HtmlParser.tcl]
 if {![winfo exists .menus]} {
     ReadSupportedLang
     ReadLangFile en
+    SetEncodings
+
+    wm geometry . 300x1
+    wm title . "Getleft v0.8.5"
 
-    wm title . "Getleft v 0.8.3"
-    if [regexp {Windows} $tcl_platform(os)] {
-         wm geometry . 300x25
-    } else {
-         wm geometry . 300x1
-    }
     . config -menu .menus
 
-    wm protocol . WM_DELETE_WINDOW exit
+    wm protocol . WM_DELETE_WINDOW {
+        SaveConfig
+        exit
+    }
 
     ReadConfig
     set options(filter)            ""
