@@ -19,10 +19,13 @@ namespace eval Dialogos {
 #    Opens a dialog window which allows the user to choose one directory. If
 #    needed, the directory is created.
 #
+# Parameter:
+#    initialDir: directory where the dialog should open itself.
+#
 # Returns
 #    The full path of the chosen directory.
 ###############################################################################
-proc SelectDirectory { } {
+proc SelectDirectory {{initialDir {} }} {
     variable fileselect
     global tcl_platform
     global dirGetleft
@@ -121,10 +124,14 @@ proc SelectDirectory { } {
     # Inicializar las variables
 
     set fileselect(path) {}
-    if {[regexp {Windows} $tcl_platform(os)]} {
-        set dir $dirGetleft(main)
+    if {$initialDir!=""} {
+        set dir $initialDir
     } else {
-        set dir $env(HOME)
+        if {[regexp {Windows} $tcl_platform(os)]} {
+            set dir $dirGetleft(main)
+        } else {
+            set dir $env(HOME)
+        }
     }
 
     set fileselect(dir) {}
