@@ -455,15 +455,11 @@ proc FilterFiles {} {
 #    link: link that we where trying to download
 ###############################################################################
 proc ErrorLog {file link} {
-    global dirGetleft env tcl_platform getleftState
+    global dirGetleft getleftState
     global labelDialogs labelMonths supportedLang options
     global errorCode
 
-    if [regexp {Windows} $tcl_platform(os)] {
-        set errorLog [file join "$dirGetleft(main)" Geterror.log]
-    } else {
-        set errorLog [file join $env(HOME) Geterror.log]
-    }
+    set errorLog [file join $dirGetleft(conf) Geterror.log]
 
     set handle  [open $errorLog a+]
     fconfigure $handle -encoding $supportedLang($options(lang))
@@ -496,13 +492,9 @@ proc ErrorLog {file link} {
 #    Deletes the error log.
 ###############################################################################
 proc EraseErrorLog {} {
-    global dirGetleft env tcl_platform
+    global dirGetleft
 
-    if [regexp {Windows} $tcl_platform(os)] {
-        set errorLog [file join "$dirGetleft(main)" Geterror.log]
-    } else {
-        set errorLog [file join $env(HOME) Geterror.log]
-    }
+    set errorLog [file join "$dirGetleft(conf)" Geterror.log]
 
     file delete $errorLog
 
@@ -515,14 +507,10 @@ proc EraseErrorLog {} {
 #    the contents of the error log.
 ###############################################################################
 proc ShowErrorLog {} {
-    global dirGetleft env tcl_platform
+    global dirGetleft
     global labelDialogs labelMonths labelTitles
 
-    if [regexp {Windows} $tcl_platform(os)] {
-        set errorLog [file join "$dirGetleft(main)" Geterror.log]
-    } else {
-        set errorLog [file join $env(HOME) Geterror.log]
-    }
+    set errorLog [file join "$dirGetleft(conf)" Geterror.log]
 
     if [file exists $errorLog] {
         Ayuda::DefineWindow $labelTitles(errorLog) $errorLog 1
