@@ -12,6 +12,120 @@
 ##############################################################################
 namespace eval HtmlParser {
 
+###############################################################################
+# SetEnttities
+#    Initializes the arrays with the translation for Html entities, something
+#    like 'entity(lt)==>'
+###############################################################################
+proc SetEntities {} {
+    variable entities
+
+        set entities(quot)      \"
+        set entities(amp)       \\&
+        set entities(lt)        <
+        set entities(gt)        >
+        set entities(nbsp)      {}
+        set entities(iexcl)     
+        set entities(cent)      
+        set entities(pound)     
+        set entities(curren)    
+        set entities(yen)       
+        set entities(brvbar)    \|
+        set entities(sect)      
+        set entities(uml)       
+        set entities(copy)      
+        set entities(ordf)      
+        set entities(laquo)     
+        set entities(not)       
+        set entities(shy)       
+        set entities(reg)       
+        set entities(macr)      
+        set entities(deg)       
+        set entities(plusmn)    
+        set entities(sup2)      
+        set entities(sup3)      
+        set entities(acute)     
+        set entities(micro)     
+        set entities(para)      
+        set entities(middot)    
+        set entities(cedil)     
+        set entities(sup1)      
+        set entities(ordm)      
+        set entities(raquo)     
+        set entities(frac14)    
+        set entities(frac12)    
+
+        set entities(frac34)    
+        set entities(iquest)    
+        set entities(ntilde)    
+        set entities(Agrave)    
+        set entities(Aacute)    
+        set entities(Acirc)     
+        set entities(Atilde)    
+        set entities(Auml)      
+        set entities(Aring)     
+        set entities(AElig)     
+        set entities(Ccedil)    
+        set entities(Egrave)    
+        set entities(Eacute)    
+        set entities(Ecirc)     
+        set entities(Euml)      
+        set entities(Igrave)    
+        set entities(Iacute)    
+        set entities(Icirc)     
+        set entities(Iuml)      
+        set entities(ETH)       
+        set entities(Ntilde)    
+        set entities(Ograve)    
+        set entities(Oacute)    
+        set entities(Ocirc)     
+        set entities(Otilde)    
+        set entities(Ouml)      
+        set entities(times)     
+        set entities(Oslash)    
+        set entities(Ugrave)    
+        set entities(Uacute)    
+        set entities(Ucirc)     
+        set entities(Uuml)      
+        set entities(Yacute)    
+        set entities(THORN)     
+        set entities(szlig)     
+        set entities(agrave)    
+        set entities(aacute)    
+        set entities(acirc)     
+        set entities(atilde)    
+        set entities(auml)      
+        set entities(aring)     
+        set entities(aelig)     
+        set entities(ccedil)    
+        set entities(egrave)    
+        set entities(eacute)    
+        set entities(ecirc)     
+        set entities(euml)      
+        set entities(igrave)    
+        set entities(iacute)    
+        set entities(icirc)     
+        set entities(iuml)      
+        set entities(eth)       
+        set entities(ntilde)    
+        set entities(ograve)    
+        set entities(oacute)    
+        set entities(ocirc)     
+        set entities(otilde)    
+        set entities(ouml)      
+        set entities(divide)    
+        set entities(oslash)    
+        set entities(ugrave)    
+        set entities(uacute)    
+        set entities(ucirc)     
+        set entities(uuml)      
+        set entities(yacute)    
+        set entities(thorn)     
+        set entities(yuml)      
+
+    return
+}
+
 ###############################################################################
 # ShowLinks
 #    Show the links found in the last preprocessed page, it's only good for
@@ -23,12 +137,13 @@ proc ShowLinks {} {
     variable linkDescription
     variable linkType
 
+    TidyDescription
     for {set i 1} {$i<$nLinks} {incr i} {
         if [info exists linkType($i)] {
-            puts "$i: $linkFile($i) [TidyDescription -$linkDescription($i)-] \
+            puts "$i: $linkFile($i) -$linkDescription($i)- \
                     -$linkType($i)"
         } else {
-            puts "$i: $linkFile($i) [TidyDescription -$linkDescription($i)-]"
+            puts "$i: $linkFile($i) -$linkDescription($i)-"
         }
     }
     return
@@ -83,44 +198,77 @@ proc TidyDir {path} {
 
 ###############################################################################
 # TidyDescription
-#    Gets the description of the link and translates it for human eyes.
+#    Translates it for human eyes the description of the links.
 #
-# Parameter:
-#    The description to translate
-#
-# Returns:
-#    The description translated
+# Side effects:
+#    The descriptions get translated.
 ###############################################################################
-proc TidyDescription {descripcion} {
+proc TidyDescription {} {
     global labelDialogs
+    variable entities
+    variable nLinks
+    variable linkFile
+    variable linkDescription
 
-    if [regexp -nocase {^(<img)} $descripcion] {
-        if {![regexp {(?:>)(.+)} $descripcion nada descriptionText]} {
-            if {![regexp -nocase {(?:ALT *= *\")([^\"]+)} $descripcion nada descrip]} {
-                regexp -nocase {(?:src *= *\"?)([^\"\ ]+)} $descripcion nada descrip
+    if {![info exists entities]} {
+        SetEntities
+    }
+
+    for {set i 1} {$i<$nLinks} {incr i} {
+        if {[regexp {^[\ \t]*$} $linkDescription($i)]} {
+            set linkDescription($i) $linkFile($i)
+            continue
+        }
+        set linkDescription($i) [string trimleft $linkDescription($i)]
+        if [regexp -nocase {^(<img)} $linkDescription($i)] {
+            if {![regexp {(?:>)(.+)} $linkDescription($i) nada descriptionText]} {
+                if {![regexp -nocase {(?:ALT *= *\")([^\"]+)}  \
+                        $linkDescription($i) nada descrip]} {
+                    regexp -nocase {(?:src *= *\"?)([^\"\ ]+)} \
+                        $linkDescription($i) nada descrip
+                }
+                set linkDescription($i) "$labelDialogs(linkImage): $descrip"
+            } else {
+                set linkDescription($i) $descriptionText
             }
-            set descripcion "$labelDialogs(linkImage): $descrip"
-        } else {
-            set descripcion $descriptionText
+        }
+        regsub -all {<.*?>} $linkDescription($i) {} linkDescription($i)
+
+        while {[regexp {(?:&)([^ ;]+)(;)?} $linkDescription($i) old entity]} {
+            regsub {#} $entity {} entity
+
+# Eventually this should be replaced wirh "string is number"
+            if {[regexp {[0-9]+} $entity]} {
+                set new [format %c $entity]
+                regsub -all {([\\])} $new {\\\1} new
+            } else {
+                if {[catch {set entities($entity)} new]} {
+                    set new ""
+                }
+            }
+            regsub -all $old $linkDescription($i) $new linkDescription($i)
         }
     }
-    regsub -all {<.*?>}      $descripcion {}   descripcion
-    regsub -all {(&quot);?}  $descripcion {\"} descripcion
-    regsub -all {&#34;?}     $descripcion {\"} descripcion
-    regsub -all {&#39;?}     $descripcion {'}  descripcion
-    regsub -all {&#153;?}    $descripcion {TM} descripcion
-    regsub -all {&#191;?}    $descripcion {} descripcion
-    regsub -all {&lt;?}      $descripcion {<}  descripcion
-    regsub -all {&gt;?}      $descripcion {>}  descripcion
-    regsub -all {(&)(amp);?} $descripcion {\1} descripcion
-    regsub -all {&ntilde;?}  $descripcion {}  descripcion
-    regsub -all {&aacute;?}  $descripcion {}  descripcion
-    regsub -all {&eacute;?}  $descripcion {}  descripcion
-    regsub -all {&iacute;?}  $descripcion {}  descripcion
-    regsub -all {&oacute;?}  $descripcion {}  descripcion
-    regsub -all {&uacute;?}  $descripcion {}  descripcion
+    return
+}
 
-    return $descripcion
+###############################################################################
+# ChangeEncoding
+#    Changes the encoding in which the description of the links are written
+#
+# Parameters:
+#    newEncoding: The encoding to use.
+###############################################################################
+proc ChangeEncoding {newEncoding} {
+    variable nLinks
+    variable linkDescription
+
+    for {set i 1} {$i<$nLinks} {incr i} {
+        set linkDescription($i) \
+                [encoding convertfrom $newEncoding $linkDescription($i)]
+    }
+
+    return
 }
 
 ###############################################################################
@@ -129,22 +277,23 @@ proc TidyDescription {descripcion} {
 #    all links.
 #
 # Parameters:
-#    file: file which contains the page to process
+#    file: file which contains the page to process.
 #
 # Returns:
 #    - '0': No errors.
 #    - '1': Couldn't open file.
 #
 # Side efects:
-#    'nLinks': number of links plus one
-#    'linkFile' keeps the links
-#    'linkDescription' the descripcion of the link
+#    'nLinks': number of links plus one.
+#    'linkFile' keeps the links.
+#    'linkDescription' the description of the link.
 ###############################################################################
 proc Parsing {file} {
     variable nLinks
     variable linkFile
     variable linkDescription
     variable linkType
+    variable pageEncoding
     global labelDialogs labelTitles
 
     if [string match $file ""] return
@@ -157,6 +306,7 @@ proc Parsing {file} {
         return 1
     }
     catch {unset linkType}
+    set pageEncoding ""
     for {set cosaOld "aaaa" ; set rep 0; set cosa [gets $leer]; set thumbnailNext 0} \
             {(![eof $leer]) || ([string compare $cosa ""])} \
             {if {![eof $leer]} {append cosa " " [gets $leer] " "} } {
@@ -171,6 +321,9 @@ proc Parsing {file} {
             set rep 0
             set cosaOld $cosa
         }
+        regexp -nocase {(?:<meta[^>]*)(?:charset=)(?:\"|')?([^\"' ]*)} $cosa \
+                nada pageEncoding
+
         if {[regexp {^\ ?<!} $cosa]} {
             while {![regexp {(?:>)(.*)} $cosa nada cosa]} {
                 append cosa " " [gets $leer]
@@ -251,7 +404,7 @@ proc Parsing {file} {
 ###############################################################################
 # RemoveLinks
 #    Removes a link from linkFile and linkDescription as ordered by procedure
-#    'FilterLinks
+#    'FilterLinks'
 #
 # Parameter
 #    index: index of the entry to be removed from the arrays.
