@@ -88,8 +88,9 @@ proc HeadRequest {link mother} {
 # It seems some servers return the body on a HEAD request
     variable workaround
 
-    set meta(content) ""
-    set meta(relocate)  ""
+    set meta(content)  ""
+    set meta(relocate) ""
+    set meta(charSet)  ""
     set workaround 0
 
     if [info exists meta(totalBytes)] {
@@ -198,24 +199,26 @@ proc Lector {pipe tipo} {
             set curlReport(end) 1
         }
         catch {close $pipe}
-        if {$errorCode!=""} {
-            set curlError [lindex $errorCode 2]
-	        if {$curlError==18} {
-                if {$::DEBUG==1} {
-                    if {$tipo!="dat"} {
-                        tk_messageBox -icon error -title error \
-                                -message "Error 18 en el cuerpo"
-                    }
-                }
-                set curlError 0
-                set errorCode ""
-            }
-            if {$::DEBUG==1} {
-                if {$curlError!=""} {
+# In the newer versions of Tcl (8.2 and 8.3) there seems to be a bug with 
+# raising errors while closing a pipe after a fileevent, so I can't trust it.
+#        if {$errorCode!=""} {
+#            set curlError [lindex $errorCode 2]
+#	        if {$curlError==18} {
+#                if {$::DEBUG==1} {
+#                    if {$tipo!="dat"} {
+#                        tk_messageBox -icon error -title error \
+#                                -message "Error 18 en el cuerpo"
+#                    }
+#                }
+#                set curlError 0
+#                set errorCode ""
+#            }
+#            if {$::DEBUG==1} {
+#                if {$curlError!=""} {
 #                    puts "Cdigo de error: $curlError - $errorCode"
-                }
-            }
-        }
+#                }
+#            }
+#        }
         return
     }
 
@@ -225,6 +228,12 @@ proc Lector {pipe tipo} {
         }
         if {[string first curl $line]!=-1} {
             set Ventana::Rizo::errorMessage $line
+            regexp {(?:\()([0-9]*)(?:\))} $line nada exitCode
+            if {($exitCode==18)&&($tipo=="cab")} {
+                set curlError 0
+            } else {
+                set curlError $exitCode
+            }
             return
         }
         if [string match $line ""] return
@@ -247,6 +256,7 @@ proc Lector {pipe tipo} {
             regexp -nocase {^(?:Last-Modified: )(.*)} $line nada meta(lastModified)
             regexp -nocase {^(?:Content-Length: )(.*)} \
                     $line nada meta(totalBytes)
+            regexp -nocase {(?:charset=)(.*)} $line nada meta(charSet)
         } else {
             set curlReport(speed) 0
             set curlReport(gotBytes)  0
@@ -290,7 +300,7 @@ proc Lector {pipe tipo} {
                             "[format "%.2f" [expr $curlReport(speed)/1024.0]] k/s"
                     } else {
                         set curlReport(speed) \
-                            "[format "%.2f" $curlReport(speed)] bytes/s"
+                            "[format "%.0f" $curlReport(speed)] bytes/s"
                     }
                 }
             }
